#ifndef __ChiliFileDecoder_h__
#define __ChiliFileDecoder_h__
#include <sndfile.h>
#include <string>
using namespace std;
class ChiliSession;
class ChiliFileDecoder {
	// A ChiliFileDecoder object encapsulates the on-disk file that is being
	// handled. As such a ChiliFileDecoder object can represent several
	// channels.
	//
	// When a ChiliFileDecoder object is constructed, this object can be used
	// to access the audio file on disk. Normally, files aren't
	// fully loaded into RAM to allow many files with a very large 
	// total filesize to be accessed.
	//
	// To allow non-destructive editing of the file, we will always
	// work on a copy of the file, which may replace the original again
	// when saving.
	// The additional advantage of this is that we can create the initial
	// copy of a file by decoding the original; thus the working copy is
	// always format-neutral. 
	//
	// We can handle decoding the file in a variety of ways.
	// - 	It may be that libsndfile or another onboard library
	//	supports the file format
	// -	It may be we have a built-in converter that can
	//	directly read the file format and convert it to
	//	something useful such as raw mono PCM
	// -	In any other case, it may be we have access to
	//	converters from one format to another, allowing
	//	an indirect conversion. For example, if we do not
	//	have native mp3 support but we have an mp3 to wav
	//	converter and we have native wav support, we can
	//	figure out a conversion strategy that allows us to
	//	read the file after all.

private:
	int random_range(int lowest_number, int highest_number);
	string generate_filename(string tempdir);
////////////////
// These are for libsndfile:
#define  SNDFILE_BUFFER_LEN	1048576 
// #define  SNDFILE_BUFFER_LEN 2048
	typedef struct
	{       char    *infilename ;
	                SF_INFO infileinfo ;
	} OptionData ;
////////////////
	
	friend class ChiliSession;
	
	ChiliSession * session;
	string filename; 	/* Name of original source file
			   will be copied here, if it is short enough */
	string tmpfilename; /* name of the working file (with path) */
	string filekey;	/* this will be the name of the working file (without path) */
	int samplerate;
	int number_of_channels;
	float * data;
	float * tmpdata;
	void decode_file(SNDFILE *infile);

	unsigned int lasterror;
	SNDFILE * outfile;
	SF_INFO sfinfoout;
	void initvars();
public:
	ChiliFileDecoder(ChiliSession * p_session,string strFileName,const int samplerate, const int channels);
	// The above constructor is used to create a new file object.

	ChiliFileDecoder(ChiliSession * p_session);	
	// The above constructor is mostly used with existing files.
	// The actual file loading and property setting is done
	// with LoadSourceFile.
	
	~ChiliFileDecoder();

	void LoadSourceFile(string filename);
	// Sets properties of the file object based on filename.
	// File must exist.
	// Return value is 0 if loading is successful, or a positive
	// value otherwise. In case of an error, the return value indicates
	// the error code.
	
	unsigned int GetNumberOfChannels();

	string GetDecodedFilename();
	string GetDecodedFilekey();
};
#endif
